######################################################################
# Project: Estimation and Inference for Generalized Geoadditive Models
# Author: Shan Yu, Guannan Wang, Li Wang, Chenhui Liu and Lijian Yang
# Date: November 2018
######################################################################
# Function: SBK_locp: Local polynomial regression with spline back fitting
# Input Arguments:
#	(1) y: the response variable
#	(2) X: covariates
# (3) alpha: the index of back fitting variable
# (4) xfixed
# (5) mhat: the rest sum of component functions
# (6) halpha: bandwidth
# (7) theta: parameter for negative binomial
# Output Arguments:
#	(1) predicted beta_k(xfixed)

source('key_functions3.R')
source('key_functionSBLL.R')
loc_fit=function(iter,y,ualpha,kalpha,thetaalpha,theta=0){
  u=ualpha[,iter]
  K=kalpha[,iter]
  Z=cbind(1,u)
  b=c(1,1)
  b_new=c(0,0)
  step=0
  while(sum((b_new-b)^2)>0.00001 & step <= 10){
    step=step+1
    b=b_new
    eta=Z%*%b+thetaalpha
    mu=linkinv(eta)
    mevg=mu.eta(eta)
    if(theta==0) var=variance(mu) else
      var=variance(mu,theta)
    Y_iter=(eta-thetaalpha)+(y-mu)/mu.eta(eta)
    W_iter=as.vector((mevg^2)/var*K)
    temp1=W_iter*Z
    temp2=W_iter*Y_iter
    temp3=crossprod(Z,temp1)
    b_new=solve(temp3,crossprod(Z,temp2))
    if(step==0) print('not convergent')
  }
  b_new[1]
}

SBK_locp=function(y,X,initial,alpha,xfixed,mhat,halpha,theta=0){
  mhatsbk=t(initial)
  n=nrow(X)
  m=length(xfixed)
  xalpha=data.matrix(X[,alpha])

  # calculate distance matrix
  Sample=matrix(rep(xalpha,each=m),ncol=m,byrow=TRUE)
  fix=matrix(rep(xfixed,each=n),nrow=n)
  ualpha=Sample-fix
  # kernel: biweight
  kalpha=15/16*((1-(ualpha/halpha)^2+abs(1-(ualpha/halpha)^2))/2)^2/halpha

  thetaalpha=rowSums(mhat[,-alpha])
  sapply(1:m,loc_fit,y=y,ualpha,kalpha,thetaalpha=thetaalpha,theta=theta)
}
