######################################################################
# Project: Estimation and Inference for Generalized Geoadditive Models
# Author: Shan Yu, Guannan Wang, Li Wang, Chenhui Liu and Lijian Yang
# Date: November 2018
######################################################################
# Function: DataGenerator_nb
# Generate dataset for the simulation example 1, case 2
# Input:
# (1) n: the sample size
# Output:
# (1) Y: the response variable
# (2) S: the location information
# (3) X: the covariate information
set.seed(1234)

DataGenerator_nb=function(n){
  # Construct the HorseShoe boundary
  fsb=list(fs.boundary())[[1]]
  tt=1
  while(tt>0){
    # Generate gridded population
    nmax=n*3
    v=runif(nmax)*5-1
    w=runif(nmax)*2-1
    m=fs.test(v,w,b=1)+sin(5*v)+cos(5*w)
    names(fsb)=c("v","w")
    ind=inSide(fsb,x=v,y=w) # remove outsiders
    v=v[ind]
    w=w[ind]
    m=m[ind]
    index=sample((1:sum(ind)),size=n,replace=FALSE)
    x1=v[index]
    x2=w[index]
    m=m[index]


    # Simulation the univariate part
    m1=m2=m3=5
    while(abs(mean(m1)) > 0.01 & abs(mean(m2)) > 0.01 & abs(mean(m3)) > 0.01){
      z1=runif(n,0,1)
      z2=runif(n,0,1)
      z3=runif(n,0,1)
      Z=cbind(z1,z2,z3)

      m1=1/2*sin(2*pi*z1)-cos(pi*z1)
      m2=4*((z2-0.5)^2-2/3*(0.5)^3)
      m3=z3-0.5
    }

    # Final dataset
    eta=m1+m2+m3+0.5*m-0.5
    mu=exp(eta)
    Y=rnbinom(n,mu=mu,size=5)
    data=cbind(Y,x1,x2,z1,z2,z3,m1,m2,m3,
               m*0.5)
    colnames(data)=c("Y","x1","x2","z1","z2","z3",'m1','m2','m3','m')
    tt=sum(is.na(data))
  }

  return(list(Y=data[,1],S=data[,2:3],X=data[,4:6]))
}
