######################################################################
# Project: Estimation and Inference for Generalized Geoadditive Models
# Author: Shan Yu, Guannan Wang, Li Wang, Chenhui Liu and Lijian Yang
# Date: November 2018
#######################################################################
# Function: setup_bps: setup parameters for different families

setup_bps=function(type,link='log',theta=NULL){
  if(link=='log') {
    linkfun=function(mu) log(mu)
    linkinv=function(eta) exp(eta)
    mu.eta=function(eta) exp(eta)
    gprime=function(mu) 1/mu
  }
  if(type=='nb'){
      variance=function(mu,theta) mu+mu^2/theta
  }else if(type=='poi'){
      variance=function(mu) mu
  }
  initialize=function(y){
    if (any(y < 0))
      stop("negative values not allowed for this 'quasi' family")
      y+0.1
  }
  structure(list(family = family, link = link,
                 linkfun = linkfun, linkinv = linkinv, variance = variance,
                 mu.eta = mu.eta, gprime = gprime,
                 initialize = initialize), class = "family")
}
